/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file analog.h
 * @brief Contains analog declarations and functions.
 * 
 * Contains functions and definitions for the use of
 * analog.
 *
 * @author Colony Project, CMU Robotics Club
 * @author Dev Gurjar
 **/

#ifndef _ANALOG_H_
#define _ANALOG_H_

#include "analog/query_analog.h"

/** @brief Analog port 0 **/
#define AN0 0x00
/** @brief Analog port 1 **/
#define AN1 0x01
/** @brief Analog port 2 **/
#define AN2 0x02
/** @brief Analog port 3 **/
#define AN3 0x03
/** @brief Analog port 4 **/
#define AN4 0x04
/** @brief Analog port 5 **/
#define AN5 0x05
/** @brief Analog port 6 **/
#define AN6 0x06
/** @brief Analog port 7 **/
#define AN7 0x07
/** @brief Analog port 8 **/
#define AN8 0x08
/** @brief Analog port 9 **/
#define AN9 0x09
/** @brief Analog port 10 **/
#define AN10 0x0A
/** @brief Analog port 11 **/
#define AN11 0x0B
/** @brief Analog port 12 **/
#define AN12 0x0C
/** @brief Analog port 13 **/
#define AN13 0x0D
/** @brief Analog port 14 **/
#define AN14 0x0E
/** @brief Analog port 15 **/
#define AN15 0x0F

/** @brief Struct to hold the value of a particular analog port **/
typedef struct
{
  uint8_t adc8;
  uint16_t adc10;
} adc_t;

/* The number of messages in the queue. If messages arrive faster than they are
 * handled, old ones are thrown out */
#define QUEUE_SIZE 10

/** @brief Initialize the analog module and driver. **/
int main(int argc, char **argv);

/** @brief given a port, returns the analog reading at that port **/
unsigned int get_sensor_data(int port);

/** @brief Responds to service to query analog input. **/
bool analog_query(analog::query_analog::Request &req,
                  analog::query_analog::Response &res);
#endif
