/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file cliffsensor.h
 * @brief Contains cliffsensor declarations and functions.
 * 
 * Contains functions and definitions for the use of
 * cliffsensors.
 *
 * @author Colony Project, CMU Robotics Club
 * @author Priyanka Deo
 **/

#ifndef _CLIFFSENSOR_H_
#define _CLIFFSENSOR_H_

#include "cliffsensor/query_cliff.h"
#include "cliffsensor/cliff_status_changed.h"

#define CLIFF_DETECTED 0x0
#define NO_CLIFF 0x1

/** @brief Initialize the cliffsensor module and driver. **/
int main(int argc, char **argv);

/** @brief Responds to topic to change cliff sensor status. **/
void changed_cliff_status(const cliffsensor::cliff_status_changed::ConstPtr& msg);

/** @brief Responds to service to query cliffsensor data. **/
bool cliff_query(cliffsensor::query_cliff::Request &req,
                  cliffsensor::query_cliff::Response &res);

#endif
