/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file headlights.cpp
 * @brief Contains code to control the headlights.
 *
 * Implementation of functions for headlights use.
 *
 * @author Colony Project, CMU Robotics Club
 **/

#include "ros/ros.h"
#include "headlights.h"
//#include "libscout/src/constants.h"
#include <cstdlib>

/**
 * @defgroup headlights headlights
 * @brief Functions for using the headlights
 *
 * @{
 **/

/* Headlight state variables
 * Each color intensity is stored as a value out of the max bunary value that 
 * can be written to the output pins that control the LEDs.
 */
/** @todo Fix types: static */
struct headlight
{
	char red;
	char green;
	char blue;
};
headlight left, right;

/**
 * @brief Sets headlight colors
 *
 * Sets the headlight colors based on subscription to the set_headlights topic.
 *
 * @param msg The message from the set_headlights topic, containing colors.
 */
void headlights_set(const headlights::set_headlights::ConstPtr& msg)
{
		/* Set state variables from message */
		if(msg->left_red > NO_SET)
		{
			left.red = msg->left_red;
		}
		if(msg->left_green > NO_SET)
		{
			left.green = msg->left_green;
		}
		if(msg->left_blue > NO_SET)
		{
			left.blue = msg->left_blue;
		}
		if(msg->right_red > NO_SET)
		{
			right.red = msg->right_red;
		}
		if(msg->right_green > NO_SET)
		{
			right.green = msg->right_green;
		}
		if(msg->right_blue > NO_SET)
		{
			right.blue = msg->right_blue;
		}
		
    /* Write colors to hardware */
    /** @todo Add code to write colors to hardware */
}

/**
 * @brief headlights driver. This is a ROS node that controls headlight colors.
 *
 * This is the main function for the headlights node. It is run when the node
 * starts and initializes the headlights. It then subscribes to the
 * set_headlights topic.
 * 
 * @param argc The number of command line arguments (should be 1)
 * @param argv The array of command line arguments
 **/
int main(int argc, char **argv)
{
    /* Initialize in ROS the headlights driver node */
    ros::init(argc, argv, "headlights_driver");

    /* Create the nodehandle for this node*/
    ros::NodeHandle n;

    /* Subscribe to the set_headlights topic */
    ros::Subscriber sub0 = n.subscribe("set_headlights", QUEUE_SIZE, headlights_set);

    /* Initialize hardware for headlights */
    // Hardware init functions here

    ROS_INFO("Ready to set headlights.");
    ros::spin();

    return 0;
}

/** @} **/
