/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file headlights.h
 * @brief Contains motor declarations and functions.
 * 
 * Contains functions and definitions for the use of
 * headlights.
 *
 * @author Colony Project, CMU Robotics Club
 **/

#ifndef _HEADLIGHTS_H_
#define _HEADLIGHTS_H_

#include "headlights/set_headlights.h"

#define FULL 0xFF
#define HALF 0xC0
#define OFF  0x00
#define NO_SET -1

/* The number of messages in the queue. If messages arrive faster than they are
 * handled, old ones are thrown out */
#define QUEUE_SIZE 4


/** @brief Initialize the headlights module and driver. **/
int main(int argc, char **argv);

/** @brief Responds to topic to set motor speeds and configs. **/
void headlights_set(const headlights::set_headlights::ConstPtr& msg);

#endif
